# Setting Up Documentation for Your TypeScript Library

Write documentation optimized for Cursor and Windsurf using markdown for the Hogbase library, a library that provides shared functionality for A/B testing and launching simple landing pages.

First, read @README.md for a description of the library.

Then, methodically traverse the codebase to document the important components that we want users to know about:

- utils/analytics.ts (Posthog analytics)
- components/ExperimentsProvider.tsx (Plug together Google Ads and different landing pages)
- hooks/useExperiment.ts (A hook to get the current variant of an experiment on per-component basis)
- hooks/usePricingSignup.ts (A hook to consistently handle pricing signup flows across different landing pages, with shared analytics and backend logic)

## 🔌 Cursor + Windsurf Optimization

Both tools can index docs if they're:

- **Markdown with semantic sections**
- **Organized per topic/concept**
- **Stored in a `docs/` folder**
- **No build step required** (like Yard or JSDoc-generated HTML — do not use these!)

You should chunk by heading, so:

- Use **`##`** or deeper headers for each topic/subtopic
- Keep section lengths ~300–500 tokens max for LLM efficiency
- Avoid mixing too many methods/classes in one file

---

## 🧠 Bonus: Embed Metadata in Markdown for Advanced Indexing

For Windsurf-style semantic search, you should add frontmatter metadata:

```markdown
---
title: MyClass
type: class
tags: [ruby, dependency injection, dsl]
summary: A dependency injection container with an expressive DSL.
---

# MyClass

...

1. Familiarize yourself with the codebase
2. `mkdir -p docs/`
3. Document each file in the `docs/` folder using the instructions above
4. Verify that the docs are correct
5. Add docusaurus
6. Generate the docusaurus site
7. Symlink the docusaurus site to the `docs/` folder
8. Convert the existing md docs to mdx and use any react you need
9. Prepare docusaurus for production:

🧭 Goal:
Auto-deploy docs/ (markdown or MDX) to GitHub Pages

Integrate cleanly with windsurf (e.g. npm run docs)

Keep docs in-repo and version-controlled

Ensure public visibility at https://yourusername.github.io/your-repo

✅ Step-by-Step Setup

1. 📁 Directory Structure
   We'll use a docs-site/ directory as the Docusaurus root, but still pull content from your existing docs/ directory.

bash
Copy
Edit
your-npm-package/
├── docs/ # ← Your markdown docs (you already have this)
├── docs-site/ # ← Docusaurus lives here
│ ├── docs/ → symlink to ../docs
│ └── docusaurus.config.js
├── package.json
└── .github/workflows/deploy-docs.yml 2. 🚀 Initialize Docusaurus
From your project root:

bash
Copy
Edit
npx create-docusaurus@latest docs-site classic
Delete docs-site/docs/ and symlink your existing docs:

bash
Copy
Edit
rm -rf docs-site/docs
ln -s ../docs docs-site/docs
Now your existing docs are used directly.

3. ⚙️ Update docusaurus.config.js
   In docs-site/docusaurus.config.js:

js
Copy
Edit
module.exports = {
title: 'Windsurf Docs',
url: 'https://yourusername.github.io',
baseUrl: '/your-repo/', // e.g. windsailor/windsurf → '/windsurf/'
organizationName: 'yourusername',
projectName: 'your-repo',
trailingSlash: false,
presets: [
[
'classic',
{
docs: {
path: 'docs',
sidebarPath: require.resolve('./sidebars.js'),
},
theme: {
customCss: require.resolve('./src/css/custom.css'),
},
},
],
],
}; 4. 🛠 Add a Deploy Script
In docs-site/package.json:

json
Copy
Edit
"scripts": {
"start": "docusaurus start",
"build": "docusaurus build",
"deploy": "docusaurus deploy"
}
In root package.json, add:

json
Copy
Edit
"scripts": {
"docs": "npm --prefix docs-site run start",
"docs:build": "npm --prefix docs-site run build",
"docs:deploy": "npm --prefix docs-site run deploy"
} 5. 🔐 Set Up GitHub Pages Deployment
Edit docs-site/docusaurus.config.js (continued):

js
Copy
Edit
module.exports = {
...
deploymentBranch: 'gh-pages',
plugins: [],
// Optional but helps:
onBrokenLinks: 'throw',
onBrokenMarkdownLinks: 'warn',
}; 6. 🪄 Create GitHub Workflow
Create .github/workflows/deploy-docs.yml:

yaml
Copy
Edit
name: Deploy Docs

on:
push:
branches: [main]

jobs:
deploy:
runs-on: ubuntu-latest
steps: - name: Checkout
uses: actions/checkout@v3

      - name: Setup Node
        uses: actions/setup-node@v3
        with:
          node-version: '18'

      - name: Install dependencies
        run: npm install && npm --prefix docs-site install

      - name: Deploy
        run: |
          git config --global user.email "actions@github.com"
          git config --global user.name "GitHub Actions"
          npm --prefix docs-site run deploy
        env:
          GIT_USER: github-actions
          GITHUB_TOKEN: ${{ secrets.GITHUB_TOKEN }}

⚠️ No gh-pages branch yet? The deploy script will create it on first run.
```
